@@ -287,23 +287,21 @@ def isoftype(x, xtype):
         except: return xtype in [_rawname(t) for t in type(x).__mro__]
     if xtype == type: return isatype(x)
     if xtype is None: return True
-    if isatype(xtype):
-        if not iterable(xtype):
-            if isinstance(xtype, Type): return xtype(x) not in (None, False)
-            if isinstance(xtype, type): return isinstance(x, xtype)
-            if callable(xtype):
-                name = _get_func_name(xtype)
-                if not name: return False
-                name = name.split('.')[-1]
-                if name.startswith('is') or name.endswith('able'):
-                    try: return xtype(x) not in (None, False)
-                    except: pass
-                return False
-            raise SystemError("Conflict between functions 'isatype' and 'isoftype', please contact the developer for more information. ")
-        if type(x) in xtype: return True
-        for xt in xtype:
-            if isoftype(x, xt): return True
-        return False
+    if not iterable(xtype):
+        if isinstance(xtype, Type): return xtype(x) not in (None, False)
+        if isinstance(xtype, type): return isinstance(x, xtype)
+        if callable(xtype):
+            name = _get_func_name(xtype)
+            if not name: return False
+            name = name.split('.')[-1]
+            if name.startswith('is') or name.endswith('able'):
+                try: return xtype(x) not in (None, False)
+                except: pass
+            return False
+        raise SystemError("Conflict between functions 'isatype' and 'isoftype', please contact the developer for more information. ")
+    if type(x) in xtype: return True
+    for xt in xtype:
+        if isoftype(x, xt): return True
     return False
 
 def isclassmethod(x):
@@ -377,28 +375,14 @@ class Type(type):
             if iterable(x): output.extend(Type.extractType(list(x)))
             elif isinstance(x, Type) and x.isunion:
                 output.extend(Type.extractType(x.types))
-            else: output.append(x)
+            elif isinstance(x, type) or callable(x) or isinstance(x, str): output.append(x)
         return output
 
     def __new__(cls, *_T, name=None, shape=tuple(), inv=False, ext=False, itypes=None):
-        if len(_T) == 1 and not isatype(_T): _T = _T[0]
-        if len(_T) > 0 and not isatype(_T): raise SyntaxError("Wrong parameter type. ")
+        nT = len(_T)
         _T = Type.extractType(_T)
-        baseT = []
-        have_basic = False
-        for t in _T:
-            if isinstance(t, type):
-                while True:
-                    if inheritable(t):
-                        if t.__bases__ and t.__bases__[0] == object:
-                            if have_basic: break
-                            else: have_basic = True
-                        if t in baseT: break
-                        baseT.append(t); break
-                    if not t.__bases__: break
-                    t = t.__bases__[0]
 
-        self = super().__new__(cls, cls.__name__ + '.' + '/'.join(map(typename, _T)), tuple(baseT), {})
+        self = super().__new__(cls, cls.__name__ + '.' + '/'.join(map(typename, _T)), tuple(t for t in _T if inheritable(t)), {})
         self.inv = inv
         self.types = _T
         self.name = name
@@ -513,6 +497,7 @@ class Type(type):
                 all([a==b or b in (None, -1) for a, b in zip(arg.shape, self.shape)]): pass
             else: return false
             if self.itemtypes != None:
+                if not iterable(arg): raise TypeHintError(str(arg) + " is not iterable. ")
                 if isinstance(self.itemtypes, dict):
                     if len(self.itemtypes) <= 0:
                         raise TypeHintError("Wrong item types after @, {} is not allowed. ")
@@ -538,7 +523,7 @@ class Type(type):
                     return false
                 elif isatype(self.itemtypes):
                     import random
-                    for _ in range(5):
+                    for _ in range(min(len(arg), 5)):
                         try: x = random.choice(arg)
                         except: x = arg.pop()
                         if not isoftype(x, self.itemtypes): return false
